#include "mboxparser.hpp"

namespace fs = boost::filesystem;

using namespace std;

/* The main function.
 * Parses the commandline options and processes the mbox file.
 */
int
main(const int argc, const char** argv) {
    // parse the command line options
    Cmdline_parser* cmd_parser = new Cmdline_parser();
    Configuration* config = cmd_parser->parse(argc, argv);
    delete cmd_parser;
    // open the mbox file
    fs::ifstream infile(config->input_file, fs::ifstream::in);
    assure(infile, config->input_file.string());
    // This counter is used for duplicate mails with no subject.
    // This is thought to be an optimization for the function that generates a
    // unique file name for the destination .eml file. By using this counter
    // the generating function does not have to scan all existing files with an
    // identical file name.
    long outf_counter = 1;
    // Here, the current Mail is stored line by line.
    deque<string> mailq;
    // This string holds the currently read line.
    string str;

    // Continue while the EOF is not yet reached
    while(!infile.eof()) {
        getline(infile, str, '\n');

        if(infile.fail() && !infile.eof()) {
            cerr << "Error while reading " << config->input_file << endl;
            exit(1);
        }

        // '\nFrom ' is the separator of mails in the mbox format
        // If this condition evaluates true, either the last mail was read or
        // the beginning of a new mail was found.
        // In both cases, the content of the mailq has to be stored as an eml
        // file.
        if(infile.eof() || str.compare(0, 5, "From ") == 0) {
            // save this mail
            if(!mailq.empty()) {
                Mail *mail = new Mail(&mailq, config);
                mail->store(config->output_dir, &outf_counter);
                delete mail;
            }
            continue; // don't store the 'From '-marker
        }

        // store the currently read line in the mailq
        mailq.push_back(str);
    }

    infile.close();

    return 0;
}

